      DOUBLE PRECISION FUNCTION dexpm1(x)
C     (dexpm1)
C     Return (exp(x) - 1), taking care to avoid subtraction loss.
C
C     This version uses minimax rational polynomials computed by Maple,
C     e.g.,
C
C         with(numapprox):
C         Digits := 20:
C         minimax((exp(x)-1-x)/x^2, x = -0.5 .. -0.25, [3,4], 1, 'err');
C         printf("%.2e\n", err):
C
C     on 5 subintervals, chosen to have accuracy adequate for IEEE 754
C     and VAX 32-bit arithmetic.
C     (28-Jun-2002)
      DOUBLE PRECISION x
      DOUBLE PRECISION ONE, ZERO
C
      DATA ZERO / 0.0D+00 /
      DATA ONE  / 1.0D+00 /
C
C     We handle the computation in three regions:
C
C     x in [-Infinity, -0.75):  exp(x) - 1
C     x in [-0.75, 0.50):       minimax rational polynomials in 5 blocks
C     x in [0.50, Infinity]:    exp(x) - 1
C
C     The central region suffers loss of one or more bits if the
C     simple formula is used.
C
C     The following IF statements handle the case of NaN, signed zero,
C     and the three regions above.
C
      IF (x .NE. x) THEN
          dexpm1 = x
      ELSE IF (x .EQ. ZERO) THEN
          dexpm1 = x
      ELSE IF (x .LT. -0.75D+00) THEN
          dexpm1 = dexp(x) - ONE
      ELSE IF (x .LT. -0.50D+00) THEN
C
C         [-0.75,-0.50) Error = 1.54e-16
C
          dexpm1 = x + ( x * x * (
     X        (  3.8989912286053430d-01 +
     X        ( -2.8882046340622964d-02 +
     X        (  5.4512449587041985d-03
     X        ) * x) * x)
     X        /
     X        (  7.7979824391446047d-01 +
     X        ( -3.1769686204763836d-01 +
     X        (  5.1818147493317685d-02 +
     X        ( -3.7949302563597960d-03 +
     X        (  7.5078784336773636d-05
     X        ) * x) * x) * x) * x) ) )
C
      ELSE IF (x .LT. -0.25D+00) THEN
C
C         [-0.50,-0.25) Error = 1.84e-16
C
          dexpm1 = x + ( x * x * (
     X        (  4.3047607933058647d-01 +
     X        ( -3.0420323905470767d-02 +
     X        (  6.0523043793850547d-03
     X        ) * x) * x)
     X        /
     X        (  8.6095215861346822d-01 +
     X        ( -3.4782470164735630d-01 +
     X        (  5.6300154414043854d-02 +
     X        ( -4.1305690642944567d-03 +
     X        (  9.0604853093734416d-05
     X        ) * x) * x) * x) * x) ) )
C
      ELSE IF (x .LT. 0.0D+00) THEN
C
C         [-0.25,-0.0) Error = 2.17e-16
C
          dexpm1 = x + ( x * x * (
     X        (  4.7542798600518708d-01 +
     X        ( -3.2253455799147504d-02 +
     X        (  6.7502300184090888d-03
     X        ) * x) * x)
     X        /
     X        (  9.5085597201037374d-01 +
     X        ( -3.8145890226858586d-01 +
     X        (  6.1415429781247393d-02 +
     X        ( -4.5311678730662972d-03 +
     X        (  1.0881543591634231d-04
     X        ) * x) * x) * x) * x) ) )
C
      ELSE IF (x .LT. 0.25D+00) THEN
C
C         [0.0,0.25) Error = 6.08e-19
C
          dexpm1 = x + ( x * x * (
     X        (  5.2593687320286602d-01 +
     X        ( -3.8811514737428555d-02 +
     X        (  8.4091491985826436d-03
     X        ) * x) * x)
     X        /
     X        (  1.0518737464057320d+00 +
     X        ( -4.2824761161010176d-01 +
     X        (  7.1911356733440284d-02 +
     X        ( -5.8143803853551491d-03 +
     X        (  1.6110243439588698d-04 +
     X        (  4.4745549016721645d-06
     X        ) * x) * x) * x) * x) * x) ) )
C
      ELSE IF (x .LT. 0.50D+00) THEN
C
C         [0.25,0.50) Error = 9.44e-19
C
          dexpm1 = x + ( x * x * (
     X        (  5.8317226176903968d-01 +
     X        ( -4.2576514699527594d-02 +
     X        (  9.2488617290439876d-03
     X        ) * x) * x)
     X        /
     X        (  1.1663445235391396d+00 +
     X        ( -4.7393453726883370d-01 +
     X        (  7.9280525808507673d-02 +
     X        ( -6.3713737409098503d-03 +
     X        (  1.7614766224757315d-04 +
     X        (  4.5318600924798633d-06
     X        ) * x) * x) * x) * x) * x) ) )
C
      ELSE
          dexpm1 = dexp(x) - ONE
      ENDIF
C
      END
