      DOUBLE PRECISION FUNCTION dexpm1(x)
C     (dexpm1)
C     Return (exp(x) - 1), taking care to avoid subtraction loss.
C     (11-Jun-2002)
      DOUBLE PRECISION x
      DOUBLE PRECISION CUTLO, CUTHI, ONE, ZERO
      DOUBLE PRECISION sum, term, xn
C
C     CUTLO = ln(0.5) (really, ln(1-1/beta) for arbitrary base beta)
C
      DATA CUTLO/-0.6931471805599453094172321214581765680755001343603D0/
C
C     CUTHI = ln(1.5) (really, ln(1+1/beta) for arbitrary base beta)
C
      DATA CUTHI/+0.4054651081081643819780131154643491365719904234625D0/
C
      DATA ZERO / 0.0D+00 /
      DATA ONE  / 1.0D+00 /
C
C     We handle the computation in three regions:
C
C     x in [-Infinity, CUTLO):  exp(x) - 1
C     x in [CUTLO, CUTHI]:      Taylor series
C     x in (CUTHI, Infinity]:   exp(x) - 1
C
C     The central region suffers loss of one or more bits if the
C     simple formula is used.
C
C     The number of terms in the Taylor series depends on the value of
C     x, and the arithmetic system: these values are for IEEE 754
C     arithmetic:
C
C                 =========================================
C                             Taylor series terms needed
C                 x         32-bit  64-bit  80-bit  128-bit
C                 =========================================
C                 CUTLO      10      17      19       29
C                 CUTLO/2     8      14      16       24
C                 0           1       1       1        1
C                 0.001       4       6       7       11
C                 0.01        4       7       9       13
C                 0.1         6      11      12       19
C                 CUTHI/2     7      12      14       22
C                 CUTHI       8      14      16       25
C                 =========================================
C
C     By using the argument reduction formula
C
C         dexpm1(x) = (exp(x) - 1)
C                   = (exp(x/2)*exp(x/2) - 1)
C                   = (dexpm1(x/2)+1)^2 - 1)
C                   = dexpm1(x/2)^2 + 2*dexpm1(x/2)
C
C     we can compute the Taylor series for x/2, then recover dexpm1(x)
C     with two multiplies and two adds.  Each Taylor series term costs
C     one add, one multiply, and one divide, and the above table
C     suggests that halving x saves two Taylor series terms.  The
C     reduction formula therefore saves two divides, which is
C     substantial.  However, this reduction should be done ONLY on
C     base-2 floating-point systems; on IBM S/390 base-16 systems,
C     division by 2 loses one bit.
C
C     The following IF statements handle the case of NaN, signed zero,
C     and the three regions above.  If NaN comparisons are botched or
C     incorrectly optimized away, code should fall through to the ELSE
C     part, which will generate a NaN provided that exp(NaN) returns a
C     NaN.  If NaN's comparisons are badly botched, then we might get
C     into a loop that never terminates: therefore, validation testing
C     is imperative!
C
      IF (x .ne. x) THEN
          dexpm1 = x
      ELSE IF (x .eq. ZERO) THEN
          dexpm1 = x
      ELSE IF (x .lt. CUTLO) THEN
          dexpm1 = dexp(x) - ONE
      ELSE IF (x .le. CUTHI) THEN
          term = x
          sum = ZERO
          xn = ONE
   10     IF ((sum + term) .ne. sum) THEN
              sum = sum + term
              xn = xn + ONE
              term = term * x / xn
              GO TO 10
          END IF
          dexpm1 = sum
      ELSE
          dexpm1 = dexp(x) - ONE
      ENDIF
C
      END
