      DOUBLE PRECISION FUNCTION dlog1p(x)
C     (log1p)
C     Return dlog(1 + x), taking care to avoid subtraction loss.
C
C     This version uses minimax rational polynomials computed by Maple,
C     e.g.,
C
C         with(numapprox):
C         Digits := 35:
C         minimax(log(1+x)/x, x = -0.5 .. -0.375, [2,2], 1, 'err');
C         printf("%.2e\n", err):
C
C     on 8 subintervals, chosen to have accuracy adequate for IEEE 754
C     and VAX 32-bit arithmetic.
C     (25-Jun-2002)
      DOUBLE PRECISION x
      DOUBLE PRECISION ONE, ZERO
C
      DATA ONE   / 1.0d+00 /
      DATA ZERO  / 0.0d+00 /
C
C     We handle the computation in three regions:
C
C     x in [-Infinity, -0.5):  log(1+x)
C     x in [-0.5, 0.5]:        minimax rational polynomials in 8 blocks
C     x in (0.5, Infinity]:    log(1+x)
C
C     The central region suffers loss of one or more bits if the
C     simple formula is used.
C
C     We also handle the cases of log1p(NaN) and log1p(0) specially,
C     so as to preserve NaNs, and the sign of zero.
C
      IF (x .ne. x) THEN
          dlog1p = x
      ELSE IF (x .eq. ZERO) THEN
          dlog1p = x
      ELSE IF (x .lt. -0.5D+00) THEN
          dlog1p = dlog(ONE + x)
      ELSE IF (x .le. -0.375D+00) THEN
C
C         Error = 1.09e-18
C
          dlog1p = x * (
     X        (  4.5777041374710203d+00 +
     X        (  1.0824898530772859d+01 +
     X        (  8.7557377520069871d+00 +
     X        (  2.7344283266106404d+00 +
     X        (  2.4810243295973145d-01
     X        ) * x) * x) * x) * x)
     X        /
     X        (  4.5777041621511807d+00 +
     X        (  1.3113751096331051d+01 +
     X        (  1.3786716041984948d+01 +
     X        (  6.4009818606418736d+00 +
     X        (  1.2159763564861204d+00 +
     X        (  6.1310243714265162d-02
     X        ) * x) * x) * x) * x) * x) )
C
      ELSE IF (x .le. -0.25D+00) THEN
C
C         Error = 1.28e-17
C
          dlog1p = x * (
     X        (  2.3664631320024130d+00 +
     X        (  4.5710195455002802d+00 +
     X        (  2.7506008228429064d+00 +
     X        (  5.1549473085824271d-01 +
     X        (  9.2891291613275938d-03
     X        ) * x) * x) * x) * x)
     X        /
     X        (  2.3664631299161469d+00 +
     X        (  5.7542510555704432d+00 +
     X        (  4.8389046840665906d+00 +
     X        (  1.6084752098772164d+00 +
     X        (  1.6581346171846399d-01
     X        ) * x) * x) * x) * x) )
C
      ELSE IF (x .le. -0.125D+00) THEN
C
C         Error = 2.61e-18
C
          dlog1p = x * (
     X        (  1.5960762199530835d+00 +
     X        (  2.9352163636366689d+00 +
     X        (  1.6546073870952289d+00 +
     X        (  2.8230129711964226d-01 +
     X        (  4.2326486667577103d-03
     X        ) * x) * x) * x) * x)
     X        /
     X        (  1.5960762199470763d+00 +
     X        (  3.7332544733236624d+00 +
     X        (  2.9892092111718665d+00 +
     X        (  9.3150673029520678d-01 +
     X        (  8.7680814287194334d-02
     X        ) * x) * x) * x) * x) )
C
      ELSE IF (x .le. 0.0D+00) THEN
C
C         Error = 6.70e-19
C
          dlog1p = x * (
     X        (  1.1512453839840972d+00 +
     X        (  2.0254530049869735d+00 +
     X        (  1.0758992666739165d+00 +
     X        (  1.6851943765528674d-01 +
     X        (  2.1555311347408590d-03
     X        ) * x) * x) * x) * x)
     X        /
     X        (  1.1512453839840972d+00 +
     X        (  2.6010756969790211d+00 +
     X        (  1.9926886538352088d+00 +
     X        (  5.8564987822829283d-01 +
     X        (  5.0770765871786014d-02
     X        ) * x) * x) * x) * x) )
C
      ELSE IF (x .le. 0.125D+00) THEN
C
C         Error = 2.03e-19
C
          dlog1p = x * (
     X        (  8.7217154834940371d-01 +
     X        (  1.4735803475030032d+00 +
     X        (  7.4104978890591803d-01 +
     X        (  1.0728917920828274d-01 +
     X        (  1.1932542494206477d-03
     X        ) * x) * x) * x) * x)
     X        /
     X        (  8.7217154834940371d-01 +
     X        (  1.9096661216777048d+00 +
     X        (  1.4051590002950247d+00 +
     X        (  3.9135619254573872d-01 +
     X        (  3.1467238052671070d-02
     X        ) * x) * x) * x) * x) )
C
      ELSE IF (x .le. 0.25D+00) THEN
C
C         Error = 7.05e-20
C
          dlog1p = x * (
     X        (  6.8586847632866200d-01 +
     X        (  1.1162704451935568d+00 +
     X        (  5.3349324796156986d-01 +
     X        (  7.1805647010229331d-02 +
     X        (  7.0487933751550271d-04
     X        ) * x) * x) * x) * x)
     X        /
     X        (  6.8586847632907489d-01 +
     X        (  1.4592046833358813d+00 +
     X        (  1.0344727647207399d+00 +
     X        (  2.7410757975888509d-01 +
     X        (  2.0561960480928906d-02
     X        ) * x) * x) * x) * x) )
C
      ELSE IF (x .le. 0.375D+00) THEN
C
C         Error = 2.71e-20
C
          dlog1p = x * (
     X        (  5.5535871709637993d-01 +
     X        (  8.7291295080225353d-01 +
     X        (  3.9774905334807039d-01 +
     X        (  5.0013874499665808d-02 +
     X        (  4.3861052898128761d-04
     X        ) * x) * x) * x) * x)
     X        /
     X        (  5.5535871712120465d-01 +
     X        (  1.1505923085672550d+00 +
     X        (  7.8792564680867556d-01 +
     X        (  1.9928550707725046d-01 +
     X        (  1.4016402522284274d-02
     X        ) * x) * x) * x) * x) )
C
      ELSE IF (x .le. 0.5D+00) THEN
C
C         Error = 1.14e-20
C
          dlog1p = x * (
     X        (  4.6035401976382008d-01 +
     X        (  7.0032345920176128d-01 +
     X        (  3.0507378915783908d-01 +
     X        (  3.5990424762222504d-02 +
     X        (  2.8479865231883178d-04
     X        ) * x) * x) * x) * x)
     X        /
     X        (  4.6035402005083824d-01 +
     X        (  9.3050046252633180d-01 +
     X        (  6.1687275191461434d-01 +
     X        (  1.4934802167217304d-01 +
     X        (  9.8909138386133843d-03
     X        ) * x) * x) * x) * x) )
C
      ELSE
          dlog1p = dlog(ONE + x)
      END IF
C
      END

      DOUBLE PRECISION FUNCTION dl1p10(x)
C     (log1p10)
C     Return dlog10(1 + x), taking care to avoid subtraction loss.
C     (17-Jun-2002)
      DOUBLE PRECISION x
      DOUBLE PRECISION dlog1p
      DOUBLE PRECISION LOG10E
C
      DATA LOG10E
     X    /0.43429448190325182765112891891660508229439700580366D+00/
C
      dl1p10 = dlog1p(x) * LOG10E
C
      END
