      DOUBLE PRECISION FUNCTION dlog1p(x)
C     (log1p)
C     Return dlog(1 + x), taking care to avoid subtraction loss.
C
C     This version uses minimax rational polynomials computed by Maple,
C     e.g.,
C
C         with(numapprox):
C         Digits := 35:
C         minimax((log(1+x)-x)/x^2, x = -0.5 .. -0.375, [4,4], 1, 'err');
C         printf("%.2e\n", err):
C
C     on 8 subintervals, chosen to have accuracy adequate for IEEE 754
C     and VAX 32-bit arithmetic.
C     (25-Jun-2002)
      DOUBLE PRECISION x
      DOUBLE PRECISION ONE, ZERO
C
      DATA ONE   / 1.0d+00 /
      DATA ZERO  / 0.0d+00 /
C
C     We handle the computation in three regions:
C
C     x in [-Infinity, -0.5):  log(1+x)
C     x in [-0.5, 0.5]:        minimax rational polynomials in 8 blocks
C     x in (0.5, Infinity]:    log(1+x)
C
C     The central region suffers loss of one or more bits if the
C     simple formula is used.
C
C     We also handle the cases of log1p(NaN) and log1p(0) specially,
C     so as to preserve NaNs, and the sign of zero.
C
      IF (x .ne. x) THEN
          dlog1p = x
      ELSE IF (x .eq. ZERO) THEN
          dlog1p = x
      ELSE IF (x .lt. -0.5D+00) THEN
          dlog1p = dlog(ONE + x)
      ELSE IF (x .le. -0.375D+00) THEN
C
C         Error = 3.61e-17
C
          dlog1p = x + ( x * x * (
     X        ( -2.1067672666150430d+00 +
     X        ( -4.2824449504183015d+00 +
     X        ( -2.7379532031064147d+00 +
     X        ( -5.4207394909776909d-01 +
     X        ( -3.3503377664620536d-03
     X        ) * x) * x) * x) * x)
     X        /
     X        (  4.2135344070215753d+00 +
     X        (  1.1373910634693916d+01 +
     X        (  1.0951729914364848d+01 +
     X        (  4.3836930292059099d+00 +
     X        (  5.9819340471291773d-01
     X        ) * x) * x) * x) * x) ) )
C
      ELSE IF (x .le. -0.25D+00) THEN
C
C         Error = 4.95e-18
C
          dlog1p = x + ( x * x * (
     X        ( -1.2555535089136690d+00 +
     X        ( -2.4231652942688882d+00 +
     X        ( -1.4488297634459479d+00 +
     X        ( -2.6133820609156552d-01 +
     X        ( -1.1648260213895755d-03
     X        ) * x) * x) * x) * x)
     X        /
     X        (  2.5111070163108593d+00 +
     X        (  6.5204018931364974d+00 +
     X        (  5.9890401685318254d+00 +
     X        (  2.2596089206732964d+00 +
     X        (  2.8533006195055604d-01
     X        ) * x) * x) * x) * x) ) )
C
      ELSE IF (x .le. -0.125D+00) THEN
C
C         Error = 9.41e-19
C
          dlog1p = x + ( x * x * (
     X        ( -8.2574484202540701d-01 +
     X        ( -1.5220596398831249d+00 +
     X        ( -8.5705710067193204d-01 +
     X        ( -1.4218241537477148d-01 +
     X        ( -4.8080284563162187d-04
     X        ) * x) * x) * x) * x)
     X        /
     X        (  1.6514896840466312d+00 +
     X        (  4.1451124022653601d+00 +
     X        (  3.6517776234049690d+00 +
     X        (  1.3069228711838949d+00 +
     X        (  1.5390276643618212d-01
     X        ) * x) * x) * x) * x) ) )
C
      ELSE IF (x .le. 0.0D+00) THEN
C
C         Error = 2.26e-19
C
          dlog1p = x + ( x * x * (
     X        ( -5.8173932923442479d-01 +
     X        ( -1.0287332510385179d+00 +
     X        ( -5.4848284009904027d-01 +
     X        ( -8.4309866797958571d-02 +
     X        ( -2.2442237938435112d-04
     X        ) * x) * x) * x) * x)
     X        /
     X        (  1.1634786584688496d+00 +
     X        (  2.8331189410562682d+00 +
     X        (  2.4039723116677090d+00 +
     X        (  8.2009993422419316d-01 +
     X        (  9.0617334974994608d-02
     X        ) * x) * x) * x) * x) ) )
C
      ELSE IF (x .le. 0.125D+00) THEN
C
C         Error = 6.47e-20
C
          dlog1p = x + ( x * x * (
     X        ( -4.3108945661266986d-01 +
     X        ( -7.3396166660196585d-01 +
     X        ( -3.7212094075155467d-01 +
     X        ( -5.3324115416485342d-02 +
     X        ( -1.1490673030384065d-04
     X        ) * x) * x) * x) * x)
     X        /
     X        (  8.6217891322533973d-01 +
     X        (  2.0427092753541580d+00 +
     X        (  1.6749586084599134d+00 +
     X        (  5.4680423074955077d-01 +
     X        (  5.6977402266381198d-02
     X        ) * x) * x) * x) * x) ) )
C
      ELSE IF (x .le. 0.25D+00) THEN
C
C         Error = 2.12e-20
C
          dlog1p = x + ( x * x * (
     X        ( -3.3202949569260634d-01 +
     X        ( -5.4584722527347350d-01 +
     X        ( -2.6410010210848144d-01 +
     X        ( -3.5461329955933539d-02 +
     X        ( -6.3238705161628870d-05
     X        ) * x) * x) * x) * x)
     X        /
     X        (  6.6405899138546901d-01 +
     X        (  1.5344004447900814d+00 +
     X        (  1.2191043387188950d+00 +
     X        (  3.8208225510585988d-01 +
     X        (  3.7703037081218612d-02
     X        ) * x) * x) * x) * x) ) )
C
      ELSE IF (x .le. 0.375D+00) THEN
C
C         Error = 7.78e-21
C
          dlog1p = x + ( x * x * (
     X        ( -2.6362222534735272d-01 +
     X        ( -4.1948059191428424d-01 +
     X        ( -1.9426196116534747d-01 +
     X        ( -2.4548408973981566d-02 +
     X        ( -3.6879920770447739d-05
     X        ) * x) * x) * x) * x)
     X        /
     X        (  5.2724445070969406d-01 +
     X        (  1.1904574838183100d+00 +
     X        (  9.1854002659347387d-01 +
     X        (  2.7712581187517214d-01 +
     X        (  2.5989498358647867d-02
     X        ) * x) * x) * x) * x) ) )
C
      ELSE IF (x .le. 0.5D+00) THEN
C
C         Error = 3.11e-21
C
          dlog1p = x + ( x * x * (
     X        ( -2.1449676782221094d-01 +
     X        ( -3.3103494044041502d-01 +
     X        ( -1.4710714256697551d-01 +
     X        ( -1.7561808331534105d-02 +
     X        ( -2.2555618919370095d-05
     X        ) * x) * x) * x) * x)
     X        /
     X        (  4.2899353581317399d-01 +
     X        (  9.4806556744940616d-01 +
     X        (  7.1176127170923616d-01 +
     X        (  2.0719548059191995d-01 +
     X        (  1.8524437616565961d-02
     X        ) * x) * x) * x) * x) ) )
C
      ELSE
          dlog1p = dlog(ONE + x)
      END IF
C
      END

      DOUBLE PRECISION FUNCTION dl1p10(x)
C     (log1p10)
C     Return dlog10(1 + x), taking care to avoid subtraction loss.
C     (17-Jun-2002)
      DOUBLE PRECISION x
      DOUBLE PRECISION dlog1p
      DOUBLE PRECISION LOG10E
C
      DATA LOG10E
     X    /0.43429448190325182765112891891660508229439700580366D+00/
C
      dl1p10 = dlog1p(x) * LOG10E
C
      END
