      DOUBLE PRECISION FUNCTION dlog1p(x)
C     (dlog1p)
C     Return dlog(1 + x), taking care to avoid subtraction loss.
C
C     This version uses a [7,6]-degree rational minimax polynomial
C     computed in Maple by
C
C         with(numapprox):
C         Digits := 50:
C         minimax((ln(1+x)-x)/x^2, x = -0.5..0.5, [7,6], 1, 'err');
C         printf("%.2e\n", err);
C
C     The reported absolute error is 7.17e-17, below the IEEE 754
C     machine epsilon of 2**(-52) = 2.22e-16.  This is the lowest
C     degree, and thus optimal, minimax polynomial whose error is below
C     that limit.
C     (24-Jun-2002)
      DOUBLE PRECISION x
      DOUBLE PRECISION CUTLO, CUTHI, ONE, ZERO
C
C     CUTLO = -1/beta, for arbitrary base beta
C
      DATA CUTLO / -0.5d+00 /
C
C     CUTHI = 1/beta, for arbitrary base beta
C
      DATA CUTHI / +0.5d+00 /
C
      DATA ONE   / 1.0d+00 /
      DATA ZERO  / 0.0d+00 /
C
C     We handle the computation in three regions:
C
C     x in [-Infinity, CUTLO):  log(1+x)
C     x in [CUTLO, CUTHI]:      minimax polynomial
C     x in (CUTHI, Infinity]:   log(1+x)
C
C     The central region suffers loss of one or more bits if the
C     simple formula is used.
C
C     We also handle the cases of dlog1p(NaN) and dlog1p(0) specially,
C     so as to preserve NaNs, and the sign of zero.
C
      IF (x .ne. x) THEN
          dlog1p = x
      ELSE IF (x .eq. ZERO) THEN
          dlog1p = x
      ELSE IF (x .lt. CUTLO) THEN
          dlog1p = dlog(ONE + x)
      ELSE IF (x .le. CUTHI) THEN
          dlog1p = x + (x * x * (
     X      ( -2.8914570958276356d-01 +
     X      ( -8.8388057507274367d-01 +
     X      ( -1.0247798215469895d+00 +
     X      ( -5.5577579361179802d-01 +
     X      ( -1.3834683248688421d-01 +
     X      ( -1.2342893701985236d-02 +
     X      ( -2.3225275641316998d-05 +
     X      (  8.3095603786418285d-07
     X      ) * x) * x) * x) * x) * x) * x) * x)
     X      /
     X      (  5.7829141916552716d-01 +
     X      (  2.1532887629225038d+00 +
     X      (  3.1959397754595289d+00 +
     X      (  2.3968502904017205d+00 +
     X      (  9.4517566962711295d-01 +
     X      (  1.8221686425529235d-01 +
     X      (  1.3015888925957095d-02
     X      ) * x) * x) * x) * x) * x) * x) ) )
      ELSE
          dlog1p = dlog(ONE + x)
      END IF
C
      END

      DOUBLE PRECISION FUNCTION dl1p10(x)
C     (log1p10)
C     Return dlog10(1 + x), taking care to avoid subtraction loss.
C     (17-Jun-2002)
      DOUBLE PRECISION x
      DOUBLE PRECISION dlog1p
      DOUBLE PRECISION LOG10E
C
      DATA LOG10E
     X /0.43429448190325182765112891891660508229439700580366D+00/
C
      dl1p10 = dlog1p(x) * LOG10E
C
      END
